package xyz.lychee.lagfixer.managers;

import com.google.common.collect.Iterators;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import java.io.OutputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.net.ssl.HttpsURLConnection;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.Logger;
import org.apache.logging.log4j.core.Filter.Result;
import org.apache.logging.log4j.core.filter.AbstractFilter;
import org.bukkit.Bukkit;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.objects.AbstractManager;
import xyz.lychee.lagfixer.objects.AbstractMonitor;

public class ErrorsManager extends AbstractManager {
   private static final Gson gson = new Gson();
   private static ErrorsManager instance;
   private final UUID uuid = UUID.randomUUID();
   private final ArrayList<String> errors = new ArrayList();
   private final Pattern pattern = Pattern.compile("https://spark\\.lucko\\.me/.{10}");
   private final AbstractFilter filter;

   public ErrorsManager(LagFixer plugin) {
      super(plugin);
      instance = this;
      this.filter = new AbstractFilter() {
         public Filter.Result filter(LogEvent event) {
            if (event.getLoggerName().equals(ErrorsManager.this.getPlugin().getLogger().getName())) {
               return Result.NEUTRAL;
            } else if (event.getThrown() != null) {
               return ErrorsManager.this.checkError(event.getThrown()) ? Result.NEUTRAL : Result.DENY;
            } else {
               Matcher matcher = ErrorsManager.this.pattern.matcher(event.getMessage().getFormattedMessage());
               if (matcher.find()) {
                  int players = Bukkit.getOnlinePlayers().size();
                  UpdaterManager updater = UpdaterManager.getInstance();
                  if (updater != null && !updater.isNeedupdate() && players > 20) {
                     AbstractMonitor monitor = SupportManager.getInstance().getMonitor();
                     ErrorsManager.this.sendError("# [Open spark profiler!](" + matcher.group() + ")", "Players: " + players + "/" + Bukkit.getMaxPlayers() + "\nCpu process: " + monitor.getCpuProcess() + "%\nCpu system: " + monitor.getCpuSystem() + "%\nRam: " + monitor.getRamUsed() + "/" + monitor.getRamTotal() + "\nTps: " + monitor.getTps() + "\nMspt: " + monitor.getMspt());
                     ErrorsManager.this.getPlugin().getLogger().info("&7Spark profiler has been sent to our support to improve LagFixer optimizations and investigate what loads the server the most.");
                  }
               }

               return Result.NEUTRAL;
            }
         }
      };
   }

   public void load() {
      Logger logger = (Logger)LogManager.getRootLogger();
      if (!Iterators.any(logger.getFilters(), (f) -> {
         return f.hashCode() == this.filter.hashCode();
      })) {
         logger.addFilter(this.filter);
      }

      this.getPlugin().getLogger().info(" &8• &rStarted listening for console LagFixer errors!");
   }

   public void disable() throws Exception {
   }

   public boolean checkError(Throwable ex) {
      if (ex == null) {
         return true;
      } else {
         String name = ExceptionUtils.getMessage(ex);
         String error = ExceptionUtils.getStackTrace(ex);
         if (!error.toLowerCase().contains("lagfixer")) {
            return true;
         } else {
            StringBuilder message = new StringBuilder();
            message.append("LagFixer error message:\n");
            message.append("&8&m-------------------------------&r\n");
            message.append("\n");
            message.append("&fAn error occurred in lagfixer: \n");
            message.append(" &7-> &c").append(name).append("\n");
            message.append("\n");
            UpdaterManager updater = UpdaterManager.getInstance();
            if (updater != null && updater.isNeedupdate()) {
               message.append("&c&lUpdate LagFixer to the latest version (").append(updater.getVersion()).append(") to avoid this error!\n");
            } else {
               message.append("&cOur support has been informed about it, it will be fixed soon.\nMake sure the LagFixer configuration is done correctly.\nIf you have any doubts, contact support: &nhttps://discord.gg/CFmzJjgZdu&r\n").append(error);
            }

            message.append('\n');
            message.append("&8&m-------------------------------\n");
            this.getPlugin().getLogger().warning(message.toString());
            if (!this.errors.contains(name)) {
               this.errors.add(name);
               this.sendError(name, error);
            }

            return false;
         }
      }
   }

   public boolean isEnabled() {
      return this.getPlugin().getConfig().getBoolean("main.errors_reporter");
   }

   public void sendError(String title, String description) {
      try {
         HttpsURLConnection conn = (HttpsURLConnection)URI.create("https://api.sakuramc.pl/error?plugin=" + this.getPlugin().getName()).toURL().openConnection();
         conn.setRequestMethod("POST");
         conn.setRequestProperty("Content-Type", "application/json");
         conn.setRequestProperty("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0");
         conn.setDoOutput(true);
         JsonObject jo = new JsonObject();
         jo.addProperty("title", title);
         jo.addProperty("description", description);
         jo.addProperty("bukkit", Bukkit.getName() + " " + Bukkit.getServer().getBukkitVersion());
         jo.addProperty("version", this.getPlugin().getDescription().getVersion());
         jo.addProperty("uuid", this.uuid.toString());
         jo.addProperty("items", SupportManager.getInstance().getItemCount());
         jo.addProperty("creatures", SupportManager.getInstance().getCreatureCount());
         jo.addProperty("projectiles", SupportManager.getInstance().getProjectileCount());
         byte[] body = gson.toJson(jo).getBytes();
         OutputStream os = conn.getOutputStream();

         try {
            os.write(body, 0, body.length);
         } catch (Throwable var10) {
            if (os != null) {
               try {
                  os.close();
               } catch (Throwable var9) {
                  var10.addSuppressed(var9);
               }
            }

            throw var10;
         }

         if (os != null) {
            os.close();
         }

         conn.getInputStream().close();
         conn.disconnect();
      } catch (Exception var11) {
      }

   }

   public UUID getUuid() {
      return this.uuid;
   }

   public ArrayList<String> getErrors() {
      return this.errors;
   }

   public Pattern getPattern() {
      return this.pattern;
   }

   public AbstractFilter getFilter() {
      return this.filter;
   }

   public static ErrorsManager getInstance() {
      return instance;
   }
}
